@@ -3,6 +3,7 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
   	if (class(data)!="gwaa.data") {
 		stop("wrong data class: should be gwaa.data")
   	}
+	checkphengen(data)
 	if (!missing(snpsubset)) data <- data[,snpsubset]
 	if (!missing(idsubset)) data <- data[idsubset,]
 	
@@ -86,27 +87,35 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
 			chi2.2df <- rep(NA,lenn) #chi2[(lenn+1):(2*lenn)];
 			out$chi2.2df <- chi2.2df
 			actdf <- chi2[(2*lenn+1):(3*lenn)];
-			if (lenn<=10 && !is.numeric(clambda)) {
-				lambda <- list()
-				lambda$estimate <- NA
-				lambda$se <- NA
-				chi2.c1df <- chi2.1df;
-			} else {
-				if (is.numeric(clambda)) {
-					lambda <- list()
-					lambda$estimate <- clambda
+			lambda <- list()
+			if (is.logical(clambda)) {
+				if (lenn<10) {
+					warning("no. observations < 10; Lambda set to 1")
+					lambda$estimate <- 1.0
 					lambda$se <- NA
-					chi2.c1df <- chi2.1df/lambda$estimate;
 				} else {
+					if (lenn<100) warning("Number of observations < 100, Lambda estimate is unreliable")
 					lambda <- estlambda(chi2.1df,plot=FALSE,prop=propPs)
-					def <- 1/lambda$estimate
-					if (def > 1 && clambda) {
-						chi2.c1df <- chi2.1df;
-					} else {
-						chi2.c1df <- def*chi2.1df;
+					if (lambda$estimate<1.0 && clambda==TRUE) {
+						warning("Lambda estimated < 1, set to 1")
+						lambda$estimate <- 1.0
+						lambda$se <- NA
 					}
 				}
+			} else {
+				if (is.numeric(clambda)) {
+					lambda$estimate <- clambda
+					lambda$se <- NA
+				} else if (is.list(clambda)) {
+					if (any(is.na(match(c("estimate","se"),names(clambda)))))
+						stop("when clambda is list, should contain estimate and se")
+					lambda <- clambda
+					lambda$se <- NA
+				} else {
+					stop("clambda should be logical, numeric, or list")
+				}
 			}
+			chi2.c1df <- chi2.1df/lambda$estimate
 			effB <- chi2[(3*lenn+1):(lenn*4)]
 			effAB <- chi2[(4*lenn+1):(lenn*5)]
 			effBB <- chi2[(5*lenn+1):(lenn*6)]
@@ -136,9 +145,9 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
 		out$Pc1df <- pr.c1df/times
 		out$Pc1df <- replace(out$Pc1df,(out$Pc1df==0),1/(1+times))
 	} else {
-		out$P1df <- 1. - pchisq(chi2.1df,1)
-		out$P2df <- 1. - pchisq(chi2.2df,actdf)
-		out$Pc1df <- 1. - pchisq(chi2.c1df,1)
+		out$P1df <- pchisq(chi2.1df,1,lower=F)
+		out$P2df <- pchisq(chi2.2df,actdf,lower=F)
+		out$Pc1df <- pchisq(chi2.c1df,1,lower=F)
 	}
 	out$lambda <- lambda
 	out$effB <- effB
